@@ -34,8 +34,15 @@ SERVICE_RENAMES = {
     'lex-runtime': 'runtime.lex',
     'mturk': 'mturk-requester',
     'resourcegroupstaggingapi': 'tagging',
+    'sagemaker-runtime': 'runtime.sagemaker',
     'lex-models': 'models.lex',
     'marketplace-entitlement': 'entitlement.marketplace',
+    'pricing': 'api.pricing',
+    'mediastore-data': 'data.mediastore',
+    'iot-jobs-data': 'data.jobs.iot',
+    'kinesis-video-media': 'kinesisvideo',
+    'kinesis-video-archived-media': 'kinesisvideo',
+    'alexaforbusiness': 'a4b',
 }
 
 BLACKLIST = [
