@@ -11,15 +11,11 @@
 # ANY KIND, either express or implied. See the License for the specific
 # language governing permissions and limitations under the License.
 from tests import unittest, mock, BaseSessionTest, create_session
-import os
-import nose
 from nose.tools import assert_equal
 
 import botocore.session
 from botocore.config import Config
-from botocore.compat import six
-from botocore.exceptions import ParamValidationError, ClientError
-from botocore.stub import Stubber
+from botocore.exceptions import ParamValidationError
 
 
 class TestS3BucketValidation(unittest.TestCase):
@@ -267,7 +263,7 @@ class TestRegionRedirect(BaseS3OperationTest):
         self.assertEqual(self.http_session_send_mock.call_count, 2)
 
         calls = [c[0][0] for c in self.http_session_send_mock.call_args_list]
-        initial_url = ('https://s3-us-west-2.amazonaws.com/foo'
+        initial_url = ('https://s3.us-west-2.amazonaws.com/foo'
                        '?encoding-type=url')
         self.assertEqual(calls[0].url, initial_url)
 
@@ -289,7 +285,7 @@ class TestRegionRedirect(BaseS3OperationTest):
 
         self.assertEqual(self.http_session_send_mock.call_count, 3)
         calls = [c[0][0] for c in self.http_session_send_mock.call_args_list]
-        initial_url = ('https://s3-us-west-2.amazonaws.com/foo'
+        initial_url = ('https://s3.us-west-2.amazonaws.com/foo'
                        '?encoding-type=url')
         self.assertEqual(calls[0].url, initial_url)
 
@@ -321,6 +317,38 @@ class TestGeneratePresigned(BaseS3OperationTest):
         }
         self.assertEqual(parts, expected)
 
+    def test_default_presign_uses_sigv2(self):
+        url = self.client.generate_presigned_url(ClientMethod='list_buckets')
+        self.assertNotIn('Algorithm=AWS4-HMAC-SHA256', url)
+
+    def test_sigv4_presign(self):
+        config = Config(signature_version='s3v4')
+        client = self.session.create_client('s3', self.region, config=config)
+        url = client.generate_presigned_url(ClientMethod='list_buckets')
+        self.assertIn('Algorithm=AWS4-HMAC-SHA256', url)
+
+    def test_sigv2_presign(self):
+        config = Config(signature_version='s3')
+        client = self.session.create_client('s3', self.region, config=config)
+        url = client.generate_presigned_url(ClientMethod='list_buckets')
+        self.assertNotIn('Algorithm=AWS4-HMAC-SHA256', url)
+
+    def test_uses_sigv4_for_unknown_region(self):
+        client = self.session.create_client('s3', 'us-west-88')
+        url = client.generate_presigned_url(ClientMethod='list_buckets')
+        self.assertIn('Algorithm=AWS4-HMAC-SHA256', url)
+
+    def test_default_presign_sigv4_in_sigv4_only_region(self):
+        client = self.session.create_client('s3', 'us-east-2')
+        url = client.generate_presigned_url(ClientMethod='list_buckets')
+        self.assertIn('Algorithm=AWS4-HMAC-SHA256', url)
+
+    def test_presign_unsigned(self):
+        config = Config(signature_version=botocore.UNSIGNED)
+        client = self.session.create_client('s3', 'us-east-2', config=config)
+        url = client.generate_presigned_url(ClientMethod='list_buckets')
+        self.assertEqual(
+            'https://s3.us-east-2.amazonaws.com/', url)
 
 
 def test_correct_url_used_for_s3():
@@ -328,21 +356,42 @@ def test_correct_url_used_for_s3():
     # we construct the expect endpoint url.
     t = S3AddressingCases(_verify_expected_endpoint_url)
 
-    # The default behavior.  DNS compatible buckets
+    # The default behavior for sigv2. DNS compatible buckets
     yield t.case(region='us-west-2', bucket='bucket', key='key',
+                 signature_version='s3',
                  expected_url='https://bucket.s3.amazonaws.com/key')
     yield t.case(region='us-east-1', bucket='bucket', key='key',
+                 signature_version='s3',
                  expected_url='https://bucket.s3.amazonaws.com/key')
     yield t.case(region='us-west-1', bucket='bucket', key='key',
+                 signature_version='s3',
                  expected_url='https://bucket.s3.amazonaws.com/key')
     yield t.case(region='us-west-1', bucket='bucket', key='key',
-                 is_secure=False,
+                 signature_version='s3', is_secure=False,
                  expected_url='http://bucket.s3.amazonaws.com/key')
 
+    # The default behavior for sigv4. DNS compatible buckets still get path
+    # style addresses.
+    yield t.case(region='us-west-2', bucket='bucket', key='key',
+                 signature_version='s3v4',
+                 expected_url=(
+                     'https://s3.us-west-2.amazonaws.com/bucket/key'))
+    yield t.case(region='us-east-1', bucket='bucket', key='key',
+                 signature_version='s3v4',
+                 expected_url='https://s3.amazonaws.com/bucket/key')
+    yield t.case(region='us-west-1', bucket='bucket', key='key',
+                 signature_version='s3v4',
+                 expected_url=(
+                     'https://s3.us-west-1.amazonaws.com/bucket/key'))
+    yield t.case(region='us-west-1', bucket='bucket', key='key',
+                 signature_version='s3v4', is_secure=False,
+                 expected_url=(
+                     'http://s3.us-west-1.amazonaws.com/bucket/key'))
+
     # If you don't have a DNS compatible bucket, we use path style.
     yield t.case(
         region='us-west-2', bucket='bucket.dot', key='key',
-        expected_url='https://s3-us-west-2.amazonaws.com/bucket.dot/key')
+        expected_url='https://s3.us-west-2.amazonaws.com/bucket.dot/key')
     yield t.case(
         region='us-east-1', bucket='bucket.dot', key='key',
         expected_url='https://s3.amazonaws.com/bucket.dot/key')
@@ -372,7 +421,7 @@ def test_correct_url_used_for_s3():
     yield t.case(
         region='us-west-2', bucket='bucket', key='key',
         s3_config=virtual_hosting,
-        expected_url='https://bucket.s3-us-west-2.amazonaws.com/key')
+        expected_url='https://bucket.s3.us-west-2.amazonaws.com/key')
     yield t.case(
         region='eu-central-1', bucket='bucket', key='key',
         s3_config=virtual_hosting,
@@ -387,7 +436,18 @@ def test_correct_url_used_for_s3():
     yield t.case(
         region='us-gov-west-1', bucket='bucket', key='key',
         s3_config=virtual_hosting,
-        expected_url='https://bucket.s3-us-gov-west-1.amazonaws.com/key')
+        expected_url='https://bucket.s3.us-gov-west-1.amazonaws.com/key')
+
+    # Test restricted regions not do virtual host by default
+    yield t.case(
+        region='us-gov-west-1', bucket='bucket', key='key',
+        signature_version='s3',
+        expected_url='https://s3.us-gov-west-1.amazonaws.com/bucket/key')
+    yield t.case(
+        region='fips-us-gov-west-1', bucket='bucket', key='key',
+        signature_version='s3',
+        expected_url='https://s3-fips-us-gov-west-1.amazonaws.com/bucket/key')
+
 
     # Test path style addressing.
     path_style = {'addressing_style': 'path'}
@@ -401,7 +461,6 @@ def test_correct_url_used_for_s3():
         customer_provided_endpoint='https://foo.amazonaws.com/',
         expected_url='https://foo.amazonaws.com/bucket/key')
 
-
     # S3 accelerate
     use_accelerate = {'use_accelerate_endpoint': True}
     yield t.case(
@@ -413,6 +472,7 @@ def test_correct_url_used_for_s3():
         region='us-west-2', bucket='bucket', key='key',
         s3_config=use_accelerate,
         expected_url='https://bucket.s3-accelerate.amazonaws.com/key')
+    # Provided endpoints still get recognized as accelerate endpoints.
     yield t.case(
         region='us-east-1', bucket='bucket', key='key',
         customer_provided_endpoint='https://s3-accelerate.amazonaws.com',
@@ -426,6 +486,21 @@ def test_correct_url_used_for_s3():
         s3_config=use_accelerate, is_secure=False,
         # Note we're using http://  because is_secure=False.
         expected_url='http://bucket.s3-accelerate.amazonaws.com/key')
+    yield t.case(
+        region='us-east-1', bucket='bucket', key='key',
+        # s3-accelerate must be the first part of the url.
+        customer_provided_endpoint='https://foo.s3-accelerate.amazonaws.com',
+        expected_url='https://foo.s3-accelerate.amazonaws.com/bucket/key')
+    yield t.case(
+        region='us-east-1', bucket='bucket', key='key',
+        # The endpoint must be an Amazon endpoint.
+        customer_provided_endpoint='https://s3-accelerate.notamazon.com',
+        expected_url='https://s3-accelerate.notamazon.com/bucket/key')
+    yield t.case(
+        region='us-east-1', bucket='bucket', key='key',
+        # Extra components must be whitelisted.
+        customer_provided_endpoint='https://s3-accelerate.foo.amazonaws.com',
+        expected_url='https://s3-accelerate.foo.amazonaws.com/bucket/key')
     # Use virtual even if path is specified for s3 accelerate because
     # path style will not work with S3 accelerate.
     yield t.case(
@@ -438,26 +513,36 @@ def test_correct_url_used_for_s3():
     use_dualstack = {'use_dualstack_endpoint': True}
     yield t.case(
         region='us-east-1', bucket='bucket', key='key',
-        s3_config=use_dualstack,
-        # Still default to virtual hosted when possible.
+        s3_config=use_dualstack, signature_version='s3',
+        # Still default to virtual hosted when possible on sigv2.
         expected_url='https://bucket.s3.dualstack.us-east-1.amazonaws.com/key')
     yield t.case(
         region='us-west-2', bucket='bucket', key='key',
-        s3_config=use_dualstack,
-        # Still default to virtual hosted when possible.
+        s3_config=use_dualstack, signature_version='s3',
+        # Still default to virtual hosted when possible on sigv2.
         expected_url='https://bucket.s3.dualstack.us-west-2.amazonaws.com/key')
+    yield t.case(
+        region='us-east-1', bucket='bucket', key='key',
+        s3_config=use_dualstack, signature_version='s3v4',
+        expected_url='https://s3.dualstack.us-east-1.amazonaws.com/bucket/key')
+    yield t.case(
+        region='us-west-2', bucket='bucket', key='key',
+        s3_config=use_dualstack, signature_version='s3v4',
+        expected_url='https://s3.dualstack.us-west-2.amazonaws.com/bucket/key')
     # Non DNS compatible buckets use path style for dual stack.
     yield t.case(
         region='us-west-2', bucket='bucket.dot', key='key',
         s3_config=use_dualstack,
         # Still default to virtual hosted when possible.
-        expected_url='https://s3.dualstack.us-west-2.amazonaws.com/bucket.dot/key')
+        expected_url=(
+            'https://s3.dualstack.us-west-2.amazonaws.com/bucket.dot/key'))
     # Supports is_secure (use_ssl=False in create_client()).
     yield t.case(
         region='us-west-2', bucket='bucket.dot', key='key', is_secure=False,
         s3_config=use_dualstack,
         # Still default to virtual hosted when possible.
-        expected_url='http://s3.dualstack.us-west-2.amazonaws.com/bucket.dot/key')
+        expected_url=(
+            'http://s3.dualstack.us-west-2.amazonaws.com/bucket.dot/key'))
 
     # Is path style is requested, we should use it, even if the bucket is
     # DNS compatible.
@@ -471,6 +556,73 @@ def test_correct_url_used_for_s3():
         # Still default to virtual hosted when possible.
         expected_url='https://s3.dualstack.us-west-2.amazonaws.com/bucket/key')
 
+    # Accelerate + dual stack
+    use_accelerate_dualstack = {
+        'use_accelerate_endpoint': True,
+        'use_dualstack_endpoint': True,
+    }
+    yield t.case(
+        region='us-east-1', bucket='bucket', key='key',
+        s3_config=use_accelerate_dualstack,
+        expected_url=(
+            'https://bucket.s3-accelerate.dualstack.amazonaws.com/key'))
+    yield t.case(
+        # Region is ignored with S3 accelerate.
+        region='us-west-2', bucket='bucket', key='key',
+        s3_config=use_accelerate_dualstack,
+        expected_url=(
+            'https://bucket.s3-accelerate.dualstack.amazonaws.com/key'))
+    # Only s3-accelerate overrides a customer endpoint.
+    yield t.case(
+        region='us-east-1', bucket='bucket', key='key',
+        s3_config=use_dualstack,
+        customer_provided_endpoint='https://s3-accelerate.amazonaws.com',
+        expected_url=(
+            'https://bucket.s3-accelerate.amazonaws.com/key'))
+    yield t.case(
+        region='us-east-1', bucket='bucket', key='key',
+        # Dualstack is whitelisted.
+        customer_provided_endpoint=(
+            'https://s3-accelerate.dualstack.amazonaws.com'),
+        expected_url=(
+            'https://bucket.s3-accelerate.dualstack.amazonaws.com/key'))
+    yield t.case(
+        region='us-east-1', bucket='bucket', key='key',
+        # Even whitelisted parts cannot be duplicated.
+        customer_provided_endpoint=(
+            'https://s3-accelerate.dualstack.dualstack.amazonaws.com'),
+        expected_url=(
+            'https://s3-accelerate.dualstack.dualstack'
+            '.amazonaws.com/bucket/key'))
+    yield t.case(
+        region='us-east-1', bucket='bucket', key='key',
+        # More than two extra parts is not allowed.
+        customer_provided_endpoint=(
+            'https://s3-accelerate.dualstack.dualstack.dualstack'
+            '.amazonaws.com'),
+        expected_url=(
+            'https://s3-accelerate.dualstack.dualstack.dualstack.amazonaws.com'
+            '/bucket/key'))
+    yield t.case(
+        region='us-east-1', bucket='bucket', key='key',
+        # Extra components must be whitelisted.
+        customer_provided_endpoint='https://s3-accelerate.foo.amazonaws.com',
+        expected_url='https://s3-accelerate.foo.amazonaws.com/bucket/key')
+    yield t.case(
+        region='us-east-1', bucket='bucket', key='key',
+        s3_config=use_accelerate_dualstack, is_secure=False,
+        # Note we're using http://  because is_secure=False.
+        expected_url=(
+            'http://bucket.s3-accelerate.dualstack.amazonaws.com/key'))
+    # Use virtual even if path is specified for s3 accelerate because
+    # path style will not work with S3 accelerate.
+    use_accelerate_dualstack['addressing_style'] = 'path'
+    yield t.case(
+        region='us-east-1', bucket='bucket', key='key',
+        s3_config=use_accelerate_dualstack,
+        expected_url=(
+            'https://bucket.s3-accelerate.dualstack.amazonaws.com/key'))
+
 
 class S3AddressingCases(object):
     def __init__(self, verify_function):
@@ -478,17 +630,17 @@ class S3AddressingCases(object):
 
     def case(self, region=None, bucket='bucket', key='key',
              s3_config=None, is_secure=True, customer_provided_endpoint=None,
-             expected_url=None):
+             expected_url=None, signature_version=None):
         return (
             self._verify, region, bucket, key, s3_config, is_secure,
-            customer_provided_endpoint, expected_url
+            customer_provided_endpoint, expected_url, signature_version
         )
 
 
 def _verify_expected_endpoint_url(region, bucket, key, s3_config,
                                   is_secure=True,
                                   customer_provided_endpoint=None,
-                                  expected_url=None):
+                                  expected_url=None, signature_version=None):
     http_response = mock.Mock()
     http_response.status_code = 200
     http_response.headers = {}
@@ -500,9 +652,10 @@ def _verify_expected_endpoint_url(region, bucket, key, s3_config,
         environ['AWS_CONFIG_FILE'] = 'no-exist-foo'
         session = create_session()
         session.config_filename = 'no-exist-foo'
-        config = None
-        if s3_config is not None:
-            config = Config(s3=s3_config)
+        config = Config(
+            signature_version=signature_version,
+            s3=s3_config
+        )
         s3 = session.create_client('s3', region_name=region, use_ssl=is_secure,
                                    config=config,
                                    endpoint_url=customer_provided_endpoint)
