@@ -10,23 +10,26 @@
 # distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
 # ANY KIND, either express or implied. See the License for the specific
 # language governing permissions and limitations under the License.
-
 import mock
 import datetime
 import json
 
-import botocore
-import botocore.auth
-from botocore.compat import six, urlparse, parse_qs
+from dateutil.tz import tzutc
 
+import botocore
+import botocore.session
+import botocore.auth
+from botocore.config import Config
 from botocore.credentials import Credentials
 from botocore.credentials import ReadOnlyCredentials
 from botocore.exceptions import NoRegionError, UnknownSignatureVersionError
 from botocore.exceptions import UnknownClientMethodError, ParamValidationError
 from botocore.exceptions import UnsupportedSignatureVersionError
 from botocore.signers import RequestSigner, S3PostPresigner, CloudFrontSigner
+from botocore.signers import generate_db_auth_token
 
 from tests import unittest
+from tests import assert_url_equal
 
 
 class BaseSignerTest(unittest.TestCase):
@@ -99,7 +102,7 @@ class TestSigner(BaseSignerTest):
         self.emitter.emit_until_response.assert_called_with(
             'choose-signer.service_name.operation_name',
             signing_name='signing_name', region_name='region_name',
-            signature_version='v4')
+            signature_version='v4', context=mock.ANY)
 
     def test_choose_signer_override(self):
         request = mock.Mock()
@@ -112,7 +115,7 @@ class TestSigner(BaseSignerTest):
             self.signer.sign('operation_name', request)
 
         auth.assert_called_with(credentials=self.fixed_credentials)
-        auth.return_value.add_auth.assert_called_with(request=request)
+        auth.return_value.add_auth.assert_called_with(request)
 
     def test_emits_before_sign(self):
         request = mock.Mock()
@@ -140,6 +143,75 @@ class TestSigner(BaseSignerTest):
 
         auth.assert_not_called()
 
+    def test_generate_url_emits_choose_signer(self):
+        request_dict = {
+            'headers': {},
+            'url': 'https://foo.com',
+            'body': b'',
+            'url_path': '/',
+            'method': 'GET',
+            'context': {}
+        }
+
+        with mock.patch.dict(botocore.auth.AUTH_TYPE_MAPS,
+                             {'v4': mock.Mock()}):
+            self.signer.generate_presigned_url(request_dict, 'operation_name')
+
+        self.emitter.emit_until_response.assert_called_with(
+            'choose-signer.service_name.operation_name',
+            signing_name='signing_name', region_name='region_name',
+            signature_version='v4-query', context=mock.ANY)
+
+    def test_choose_signer_passes_context(self):
+        request = mock.Mock()
+        request.context = {'foo': 'bar'}
+
+        with mock.patch.dict(botocore.auth.AUTH_TYPE_MAPS,
+                             {'v4': mock.Mock()}):
+            self.signer.sign('operation_name', request)
+
+        self.emitter.emit_until_response.assert_called_with(
+            'choose-signer.service_name.operation_name',
+            signing_name='signing_name', region_name='region_name',
+            signature_version='v4', context={'foo': 'bar'})
+
+    def test_generate_url_choose_signer_override(self):
+        request_dict = {
+            'headers': {},
+            'url': 'https://foo.com',
+            'body': b'',
+            'url_path': '/',
+            'method': 'GET',
+            'context': {}
+        }
+        auth = mock.Mock()
+        auth.REQUIRES_REGION = False
+        self.emitter.emit_until_response.return_value = (None, 'custom')
+
+        auth_types_map = {'custom': mock.Mock(), 'custom-query': auth}
+        with mock.patch.dict(botocore.auth.AUTH_TYPE_MAPS, auth_types_map):
+            self.signer.generate_presigned_url(request_dict, 'operation_name')
+
+        auth.assert_called_with(credentials=self.fixed_credentials,
+                                expires=3600)
+
+    def test_generate_url_unsigned(self):
+        request_dict = {
+            'headers': {},
+            'url': 'https://foo.com',
+            'body': b'',
+            'url_path': '/',
+            'method': 'GET',
+            'context': {}
+        }
+        self.emitter.emit_until_response.return_value = (
+            None, botocore.UNSIGNED)
+
+        url = self.signer.generate_presigned_url(
+            request_dict, 'operation_name')
+
+        self.assertEqual(url, 'https://foo.com')
+
     def test_generate_presigned_url(self):
         auth = mock.Mock()
         auth.REQUIRES_REGION = True
@@ -149,11 +221,13 @@ class TestSigner(BaseSignerTest):
             'url': 'https://foo.com',
             'body': b'',
             'url_path': '/',
-            'method': 'GET'
+            'method': 'GET',
+            'context': {}
         }
         with mock.patch.dict(botocore.auth.AUTH_TYPE_MAPS,
                              {'v4-query': auth}):
-            presigned_url = self.signer.generate_presigned_url(request_dict)
+            presigned_url = self.signer.generate_presigned_url(
+                request_dict, operation_name='operation_name')
         auth.assert_called_with(
             credentials=self.fixed_credentials, region_name='region_name',
             service_name='signing_name', expires=3600)
@@ -168,12 +242,14 @@ class TestSigner(BaseSignerTest):
             'url': 'https://foo.com',
             'body': b'',
             'url_path': '/',
-            'method': 'GET'
+            'method': 'GET',
+            'context': {}
         }
         with mock.patch.dict(botocore.auth.AUTH_TYPE_MAPS,
                              {'v4-query': auth}):
             presigned_url = self.signer.generate_presigned_url(
-                request_dict, region_name='us-west-2')
+                request_dict, operation_name='operation_name',
+                region_name='us-west-2')
         auth.assert_called_with(
             credentials=self.fixed_credentials, region_name='us-west-2',
             service_name='signing_name', expires=3600)
@@ -188,49 +264,34 @@ class TestSigner(BaseSignerTest):
             'url': 'https://foo.com',
             'body': b'',
             'url_path': '/',
-            'method': 'GET'
+            'method': 'GET',
+            'context': {}
         }
         with mock.patch.dict(botocore.auth.AUTH_TYPE_MAPS,
                              {'v4-query': auth}):
             presigned_url = self.signer.generate_presigned_url(
-                request_dict, expires_in=900)
+                request_dict, operation_name='operation_name', expires_in=900)
         auth.assert_called_with(
             credentials=self.fixed_credentials,
             region_name='region_name',
             expires=900, service_name='signing_name')
         self.assertEqual(presigned_url, 'https://foo.com')
 
-    def test_generate_presigned_url_fixes_s3_host(self):
-        self.signer = RequestSigner(
-            'service_name', 'region_name', 'signing_name',
-            's3', self.credentials, self.emitter)
-
-        auth = mock.Mock()
-        auth.REQUIRES_REGION = True
-
+    def test_presigned_url_throws_unsupported_signature_error(self):
         request_dict = {
             'headers': {},
             'url': 'https://s3.amazonaws.com/mybucket/myobject',
             'body': b'',
             'url_path': '/',
-            'method': 'GET'
+            'method': 'GET',
+            'context': {}
         }
-        with mock.patch.dict(botocore.auth.AUTH_TYPE_MAPS,
-                             {'s3-query': auth}):
-            presigned_url = self.signer.generate_presigned_url(
-                request_dict, expires_in=900)
-        auth.assert_called_with(
-            credentials=self.fixed_credentials, region_name='region_name',
-            expires=900, service_name='signing_name')
-        self.assertEqual(presigned_url,
-                         'https://mybucket.s3.amazonaws.com/myobject')
-
-    def test_presigned_url_throws_unsupported_signature_error(self):
         self.signer = RequestSigner(
             'service_name', 'region_name', 'signing_name',
             'foo', self.credentials, self.emitter)
         with self.assertRaises(UnsupportedSignatureVersionError):
-            self.signer.generate_presigned_url({})
+            self.signer.generate_presigned_url(
+                request_dict, operation_name='foo')
 
     def test_signer_with_refreshable_credentials_gets_credential_set(self):
         class FakeCredentials(Credentials):
@@ -273,9 +334,167 @@ class TestSigner(BaseSignerTest):
                 credentials=None,
             )
 
+    def test_sign_with_signing_type_standard(self):
+        auth = mock.Mock()
+        post_auth = mock.Mock()
+        query_auth = mock.Mock()
+        request = mock.Mock()
+        auth_types = {
+            'v4-presign-post': post_auth,
+            'v4-query': query_auth,
+            'v4': auth
+        }
+        with mock.patch.dict(botocore.auth.AUTH_TYPE_MAPS, auth_types):
+            self.signer.sign('operation_name', request,
+                             signing_type='standard')
+        self.assertFalse(post_auth.called)
+        self.assertFalse(query_auth.called)
+        auth.assert_called_with(
+            credentials=ReadOnlyCredentials('key', 'secret', None),
+            service_name='signing_name',
+            region_name='region_name'
+        )
 
-class TestCloudfrontSigner(unittest.TestCase):
+    def test_sign_with_signing_type_presign_url(self):
+        auth = mock.Mock()
+        post_auth = mock.Mock()
+        query_auth = mock.Mock()
+        request = mock.Mock()
+        auth_types = {
+            'v4-presign-post': post_auth,
+            'v4-query': query_auth,
+            'v4': auth
+        }
+        with mock.patch.dict(botocore.auth.AUTH_TYPE_MAPS, auth_types):
+            self.signer.sign('operation_name', request,
+                             signing_type='presign-url')
+        self.assertFalse(post_auth.called)
+        self.assertFalse(auth.called)
+        query_auth.assert_called_with(
+            credentials=ReadOnlyCredentials('key', 'secret', None),
+            service_name='signing_name',
+            region_name='region_name'
+        )
+
+    def test_sign_with_signing_type_presign_post(self):
+        auth = mock.Mock()
+        post_auth = mock.Mock()
+        query_auth = mock.Mock()
+        request = mock.Mock()
+        auth_types = {
+            'v4-presign-post': post_auth,
+            'v4-query': query_auth,
+            'v4': auth
+        }
+        with mock.patch.dict(botocore.auth.AUTH_TYPE_MAPS, auth_types):
+            self.signer.sign('operation_name', request,
+                             signing_type='presign-post')
+        self.assertFalse(auth.called)
+        self.assertFalse(query_auth.called)
+        post_auth.assert_called_with(
+            credentials=ReadOnlyCredentials('key', 'secret', None),
+            service_name='signing_name',
+            region_name='region_name'
+        )
+
+    def test_sign_with_region_name(self):
+        request = mock.Mock()
+        auth = mock.Mock()
+        auth_types = {
+            'v4': auth
+        }
+        with mock.patch.dict(botocore.auth.AUTH_TYPE_MAPS, auth_types):
+            self.signer.sign('operation_name', request, region_name='foo')
+        auth.assert_called_with(
+            credentials=ReadOnlyCredentials('key', 'secret', None),
+            service_name='signing_name',
+            region_name='foo'
+        )
+
+    def test_sign_with_expires_in(self):
+        request = mock.Mock()
+        auth = mock.Mock()
+        auth_types = {
+            'v4': auth
+        }
+        with mock.patch.dict(botocore.auth.AUTH_TYPE_MAPS, auth_types):
+            self.signer.sign('operation_name', request, expires_in=2)
+        auth.assert_called_with(
+            credentials=ReadOnlyCredentials('key', 'secret', None),
+            service_name='signing_name',
+            region_name='region_name',
+            expires=2
+        )
+
+    def test_sign_with_custom_signing_name(self):
+        request = mock.Mock()
+        auth = mock.Mock()
+        auth_types = {
+            'v4': auth
+        }
+        with mock.patch.dict(botocore.auth.AUTH_TYPE_MAPS, auth_types):
+            self.signer.sign('operation_name', request, signing_name='foo')
+        auth.assert_called_with(
+            credentials=ReadOnlyCredentials('key', 'secret', None),
+            service_name='foo',
+            region_name='region_name'
+        )
+
+    def test_presign_with_custom_signing_name(self):
+        auth = mock.Mock()
+        auth.REQUIRES_REGION = True
+
+        request_dict = {
+            'headers': {},
+            'url': 'https://foo.com',
+            'body': b'',
+            'url_path': '/',
+            'method': 'GET',
+            'context': {}
+        }
+        with mock.patch.dict(botocore.auth.AUTH_TYPE_MAPS,
+                             {'v4-query': auth}):
+            presigned_url = self.signer.generate_presigned_url(
+                request_dict, operation_name='operation_name',
+                signing_name='foo')
+        auth.assert_called_with(
+            credentials=self.fixed_credentials,
+            region_name='region_name',
+            expires=3600, service_name='foo')
+        self.assertEqual(presigned_url, 'https://foo.com')
+
+    def test_unknown_signer_raises_unknown_on_standard(self):
+        request = mock.Mock()
+        auth = mock.Mock()
+        auth_types = {
+            'v4': auth
+        }
+        self.emitter.emit_until_response.return_value = (None, 'custom')
+        with mock.patch.dict(botocore.auth.AUTH_TYPE_MAPS, auth_types):
+            with self.assertRaises(UnknownSignatureVersionError):
+                self.signer.sign('operation_name', request,
+                                 signing_type='standard')
+
+    def test_unknown_signer_raises_unsupported_when_not_standard(self):
+        request = mock.Mock()
+        auth = mock.Mock()
+        auth_types = {
+            'v4': auth
+        }
+        self.emitter.emit_until_response.return_value = (None, 'custom')
+        with mock.patch.dict(botocore.auth.AUTH_TYPE_MAPS, auth_types):
+            with self.assertRaises(UnsupportedSignatureVersionError):
+                self.signer.sign('operation_name', request,
+                                 signing_type='presign-url')
+
+            with self.assertRaises(UnsupportedSignatureVersionError):
+                self.signer.sign('operation_name', request,
+                                 signing_type='presign-post')
+
+
+class TestCloudfrontSigner(BaseSignerTest):
     def setUp(self):
+        super(TestCloudfrontSigner, self).setUp()
         self.signer = CloudFrontSigner("MY_KEY_ID", lambda message: b'signed')
         # It helps but the long string diff will still be slightly different on
         # Python 2.6/2.7/3.x. We won't soly rely on that anyway, so it's fine.
@@ -306,20 +525,6 @@ class TestCloudfrontSigner(unittest.TestCase):
         }
         self.assertEqual(json.loads(policy), expected)
 
-    def _urlparse(self, url):
-        if isinstance(url, six.binary_type):
-            # Not really necessary, but it helps to reduce noise on Python 2.x
-            url = url.decode('utf8')
-        return dict(urlparse(url)._asdict())  # Needs an unordered dict here
-
-    def assertEqualUrl(self, url1, url2):
-        # We compare long urls by their dictionary parts
-        parts1 = self._urlparse(url1)
-        parts2 = self._urlparse(url2)
-        self.assertEqual(
-            parse_qs(parts1.pop('query')), parse_qs(parts2.pop('query')))
-        self.assertEqual(parts1, parts2)
-
     def test_generate_presign_url_with_expire_time(self):
         signed_url = self.signer.generate_presigned_url(
             'http://test.com/foo.txt',
@@ -327,7 +532,7 @@ class TestCloudfrontSigner(unittest.TestCase):
         expected = (
             'http://test.com/foo.txt?Expires=1451606400&Signature=c2lnbmVk'
             '&Key-Pair-Id=MY_KEY_ID')
-        self.assertEqualUrl(signed_url, expected)
+        assert_url_equal(signed_url, expected)
 
     def test_generate_presign_url_with_custom_policy(self):
         policy = self.signer.build_policy(
@@ -344,7 +549,7 @@ class TestCloudfrontSigner(unittest.TestCase):
             '6IjEyLjM0LjU2Ljc4LzkifSwiRGF0ZUdyZWF0ZXJUaGFuIjp7IkFX'
             'UzpFcG9jaFRpbWUiOjE0NDg5MjgwMDB9fX1dfQ__'
             '&Signature=c2lnbmVk&Key-Pair-Id=MY_KEY_ID')
-        self.assertEqualUrl(signed_url, expected)
+        assert_url_equal(signed_url, expected)
 
 
 class TestS3PostPresigner(BaseSignerTest):
@@ -359,7 +564,8 @@ class TestS3PostPresigner(BaseSignerTest):
             'url': 'https://s3.amazonaws.com/mybucket',
             'body': b'',
             'url_path': '/',
-            'method': 'POST'
+            'method': 'POST',
+            'context': {}
         }
         self.auth = mock.Mock()
         self.auth.REQUIRES_REGION = True
@@ -396,6 +602,63 @@ class TestS3PostPresigner(BaseSignerTest):
                          'https://s3.amazonaws.com/mybucket')
         self.assertEqual(post_form_args['fields'], {})
 
+    def test_generate_presigned_post_emits_choose_signer(self):
+        with mock.patch.dict(botocore.auth.AUTH_TYPE_MAPS,
+                             {'s3v4-presign-post': self.auth}):
+            self.signer.generate_presigned_post(self.request_dict)
+        self.emitter.emit_until_response.assert_called_with(
+            'choose-signer.service_name.PutObject',
+            signing_name='signing_name', region_name='region_name',
+            signature_version='s3v4-presign-post', context=mock.ANY)
+
+    def test_generate_presigned_post_choose_signer_override(self):
+        auth = mock.Mock()
+        self.emitter.emit_until_response.return_value = (None, 'custom')
+        auth_types = {
+            's3v4-presign-post': self.auth,
+            'custom-presign-post': auth
+        }
+        with mock.patch.dict(botocore.auth.AUTH_TYPE_MAPS, auth_types):
+            self.signer.generate_presigned_post(self.request_dict)
+        auth.assert_called_with(
+            credentials=self.fixed_credentials, region_name='region_name',
+            service_name='signing_name')
+
+    def test_generate_presigne_post_choose_signer_override_known(self):
+        auth = mock.Mock()
+        self.emitter.emit_until_response.return_value = (
+            None, 's3v4-presign-post')
+        auth_types = {
+            's3v4-presign-post': self.auth,
+            'custom-presign-post': auth
+        }
+        with mock.patch.dict(botocore.auth.AUTH_TYPE_MAPS, auth_types):
+            self.signer.generate_presigned_post(self.request_dict)
+        self.auth.assert_called_with(
+            credentials=self.fixed_credentials, region_name='region_name',
+            service_name='signing_name')
+
+    def test_generate_presigned_post_bad_signer_raises_error(self):
+        auth = mock.Mock()
+        self.emitter.emit_until_response.return_value = (None, 's3-query')
+        auth_types = {
+            's3v4-presign-post': self.auth,
+            's3-query': auth
+        }
+        with mock.patch.dict(botocore.auth.AUTH_TYPE_MAPS, auth_types):
+            with self.assertRaises(UnsupportedSignatureVersionError):
+                self.signer.generate_presigned_post(self.request_dict)
+
+    def test_generate_unsigned_post(self):
+        self.emitter.emit_until_response.return_value = (
+            None, botocore.UNSIGNED)
+        with mock.patch.dict(botocore.auth.AUTH_TYPE_MAPS,
+                             {'s3v4-presign-post': self.auth}):
+            post_form_args = self.signer.generate_presigned_post(
+                self.request_dict)
+        expected = {'fields': {}, 'url': 'https://s3.amazonaws.com/mybucket'}
+        self.assertEqual(post_form_args, expected)
+
     def test_generate_presigned_post_with_conditions(self):
         conditions = [
             {'bucket': 'mybucket'},
@@ -422,29 +685,21 @@ class TestS3PostPresigner(BaseSignerTest):
             credentials=self.fixed_credentials, region_name='foo',
             service_name='signing_name')
 
-    def test_generate_presigned_post_fixes_s3_host(self):
-        self.request_signer = RequestSigner(
-            'service_name', 'region_name', 'signing_name',
-            's3', self.credentials, self.emitter)
-        self.signer = S3PostPresigner(self.request_signer)
-
-        with mock.patch.dict(botocore.auth.AUTH_TYPE_MAPS,
-                             {'s3-presign-post': self.auth}):
-            post_form_args = self.signer.generate_presigned_post(
-                self.request_dict)
-        self.auth.assert_called_with(
-            credentials=self.fixed_credentials, region_name='region_name',
-            service_name='signing_name')
-        self.assertEqual(post_form_args['url'],
-                         'https://mybucket.s3.amazonaws.com/')
-
     def test_presigned_post_throws_unsupported_signature_error(self):
+        request_dict = {
+            'headers': {},
+            'url': 'https://s3.amazonaws.com/mybucket/myobject',
+            'body': b'',
+            'url_path': '/',
+            'method': 'POST',
+            'context': {}
+        }
         self.request_signer = RequestSigner(
             'service_name', 'region_name', 'signing_name',
             'foo', self.credentials, self.emitter)
         self.signer = S3PostPresigner(self.request_signer)
         with self.assertRaises(UnsupportedSignatureVersionError):
-            self.signer.generate_presigned_post({})
+            self.signer.generate_presigned_post(request_dict)
 
 
 class TestGenerateUrl(unittest.TestCase):
@@ -471,9 +726,11 @@ class TestGenerateUrl(unittest.TestCase):
             'headers': {},
             'query_string': {},
             'url_path': u'/mybucket/mykey',
-            'method': u'GET'}
+            'method': u'GET',
+            'context': {}}
         self.generate_url_mock.assert_called_with(
-            request_dict=ref_request_dict, expires_in=3600)
+            request_dict=ref_request_dict, expires_in=3600,
+            operation_name='GetObject')
 
     def test_generate_presigned_url_with_query_string(self):
         disposition = 'attachment; filename="download.jpg"'
@@ -490,9 +747,11 @@ class TestGenerateUrl(unittest.TestCase):
             'headers': {},
             'query_string': {u'response-content-disposition': disposition},
             'url_path': u'/mybucket/mykey',
-            'method': u'GET'}
+            'method': u'GET',
+            'context': {}}
         self.generate_url_mock.assert_called_with(
-            request_dict=ref_request_dict, expires_in=3600)
+            request_dict=ref_request_dict, expires_in=3600,
+            operation_name='GetObject')
 
     def test_generate_presigned_url_unknown_method_name(self):
         with self.assertRaises(UnknownClientMethodError):
@@ -512,9 +771,11 @@ class TestGenerateUrl(unittest.TestCase):
             'headers': {},
             'query_string': {},
             'url_path': u'/mybucket/mykey',
-            'method': u'GET'}
+            'method': u'GET',
+            'context': {}}
         self.generate_url_mock.assert_called_with(
-            request_dict=ref_request_dict, expires_in=20)
+            request_dict=ref_request_dict, expires_in=20,
+            operation_name='GetObject')
 
     def test_generate_presigned_url_override_http_method(self):
         self.client.generate_presigned_url(
@@ -526,9 +787,11 @@ class TestGenerateUrl(unittest.TestCase):
             'headers': {},
             'query_string': {},
             'url_path': u'/mybucket/mykey',
-            'method': u'PUT'}
+            'method': u'PUT',
+            'context': {}}
         self.generate_url_mock.assert_called_with(
-            request_dict=ref_request_dict, expires_in=3600)
+            request_dict=ref_request_dict, expires_in=3600,
+            operation_name='GetObject')
 
 
 class TestGeneratePresignedPost(unittest.TestCase):
@@ -620,3 +883,52 @@ class TestGeneratePresignedPost(unittest.TestCase):
         self.client = self.session.create_client('ec2', 'us-west-2')
         with self.assertRaises(AttributeError):
             self.client.generate_presigned_post()
+
+
+class TestGenerateDBAuthToken(BaseSignerTest):
+    maxDiff = None
+
+    def setUp(self):
+        self.session = botocore.session.get_session()
+        self.client = self.session.create_client(
+            'rds', region_name='us-east-1', aws_access_key_id='akid',
+            aws_secret_access_key='skid', config=Config(signature_version='v4')
+        )
+
+    def test_generate_db_auth_token(self):
+        hostname = 'prod-instance.us-east-1.rds.amazonaws.com'
+        port = 3306
+        username = 'someusername'
+        clock = datetime.datetime(2016, 11, 7, 17, 39, 33, tzinfo=tzutc())
+
+        with mock.patch('datetime.datetime') as dt:
+            dt.utcnow.return_value = clock
+            result = generate_db_auth_token(
+                self.client, hostname, port, username)
+
+        expected_result = (
+            'prod-instance.us-east-1.rds.amazonaws.com:3306/?Action=connect'
+            '&DBUser=someusername&X-Amz-Algorithm=AWS4-HMAC-SHA256'
+            '&X-Amz-Date=20161107T173933Z&X-Amz-SignedHeaders=host'
+            '&X-Amz-Expires=900&X-Amz-Credential=akid%2F20161107%2F'
+            'us-east-1%2Frds-db%2Faws4_request&X-Amz-Signature'
+            '=d1138cdbc0ca63eec012ec0fc6c2267e03642168f5884a7795320d4c18374c61'
+        )
+
+        # A scheme needs to be appended to the beginning or urlsplit may fail
+        # on certain systems.
+        assert_url_equal(
+            'https://' + result, 'https://' + expected_result)
+
+    def test_custom_region(self):
+        hostname = 'host.us-east-1.rds.amazonaws.com'
+        port = 3306
+        username = 'mySQLUser'
+        region = 'us-west-2'
+        result = generate_db_auth_token(
+            self.client, hostname, port, username, Region=region)
+
+        self.assertIn(region, result)
+        # The hostname won't be changed even if a different region is specified
+        self.assertIn(hostname, result)
+
